/* Copyright (c) 2017 Asustor Inc. All rights reserved. */

/**
 * vtype of radius client IP
 */
Ext.form.VTypes.radiusclientipv4Mask = /[0-9\.]/;
Ext.form.VTypes.radiusclientipv4Re = AS.ARC.re.ipv4;
Ext.form.VTypes.radiusclientipv4 = function (v, f) {
	var isNewIP = (f.old !== v);

	if (typeof f.ajaxError === 'undefined') {
		f.ajaxError = true;
		f.ajaxErrorText = '';
	}

	if (!Ext.form.VTypes.radiusclientipv4Re.test(v)) {
		Ext.form.VTypes.radiusclientipv4Text = _S('MESSAGE', 'HF00002');
		return false;
	} else if (v !== '' && (f.workName !== v)) {
		f.workName = v;
		f.ajaxError = false;
		if (!isNewIP) {
			return true;
		}
		/**
		 * Check if IP exist
		 * @param {String} act: action value.
		 * @param {String} tab: tab name.
		 * @param {String} ip: IP would be checked.
		 */
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(AS.ARC.util.getUserAppsPath() + 'radiusserver/radiusserver.cgi', {
				"tab": 'client',
				"act": 'checkip'
			}),
			method: 'post',
			params: { "ip": v },
			success: function (json, opts) {
				if (v === f.workName) {
					if (json.alreadyexists) {
						f.ajaxError = true;
						f.ajaxErrorText = _S('MESSAGE', 'HN00009');
						f.markInvalid(f.ajaxErrorText);
					} else {
						f.clearInvalid();
						f.ajaxError = false;
						f.ajaxErrorText = '';
					}
				}
			},
			failure: function (json, opts) {
				if (f.workName === v) {
					f.ajaxError = true;
					
					if (json.error_code === '5008') {
						f.ajaxErrorText = _S('MESSAGE', 'HN00009');
					} else {
						f.ajaxErrorText = 'Ref. ' + json.error_code;
					}
					f.markInvalid(f.ajaxErrorText);
				}
			}
		});
		return true;
	}
	
	if (f.ajaxError && f.ajaxErrorText != '') {
		Ext.form.VTypes.radiusclientipv4Text = f.ajaxErrorText;
		return false;
	}
	
	return true;
	
};

/**
 * vtype of radius client name
 */
Ext.form.VTypes.radiusclientnameMask = /[^`#$%\*+=\[\]|\\:;"'<>,?\/]/;
Ext.form.VTypes.radiusclientnameText = '';
Ext.form.VTypes.radiusclientname = function(v, f) {
	var MAX_LENGTH = 32,
		isNewName = (f.old !== v);

	if (typeof f.ajaxError === 'undefined') {
		f.ajaxError = true;
		f.ajaxErrorText = '';
	}
	
	if (!AS.ARC.re.userName.test(v)) { // check name format
		Ext.form.VTypes.radiusclientnameText = Ext.String.format(_S('MESSAGE', 'HF00022'), ' # $ % * + = [ ] | \\ : ; " \' < > , ? / ') + '<br>' + 
			Ext.String.format(_S('MESSAGE', 'HF00021'), '@ -') + '<br>' + 
			_S('MESSAGE', 'HF00020');
		return false;
		
	} else if (AS.ARC.util.getStringByteLength(v) > MAX_LENGTH) { // check name length 
		Ext.form.VTypes.radiusclientnameText = Ext.String.format(_S('ERROR', 'MAX_N_BYTE_LENGTH'), MAX_LENGTH);
		return false;
		
	} else if (v !== '' && (f.workName !== v)) {
		f.workName = v;
		f.ajaxError = false;
		if (!isNewName) {
			return true;
		}
		/**
		 * Check if name exist
		 * @param {String} act: action value.
		 * @param {String} tab: tab name.
		 * @param {String} name: Client name would be checked.
		 */
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(AS.ARC.util.getUserAppsPath() + 'radiusserver/radiusserver.cgi', {
				"tab": 'client',
				"act": 'check'
			}),
			method: 'post',
			params: { "name": v },
			success: function (json, opts) {
				if (v === f.workName) {
					if (json.alreadyexists) {
						f.ajaxError = true;
						f.ajaxErrorText = _S('MESSAGE', 'HN00002');
						f.markInvalid(f.ajaxErrorText);
					} else {
						f.clearInvalid();
						f.ajaxError = false;
						f.ajaxErrorText = '';
					}
				}
			},
			failure: function (json, opts) {
				if (f.workName === v) {
					f.ajaxError = true;
					
					if (json.error_code === '5008') {
						f.ajaxErrorText = _S('MESSAGE', 'HN00002');
					} else {
						f.ajaxErrorText = 'Ref. ' + json.error_code;
					}
					f.markInvalid(f.ajaxErrorText);
				}
			}
		});
		return true;
	}
	
	if (f.ajaxError && f.ajaxErrorText != '') {
		Ext.form.VTypes.radiusclientnameText = f.ajaxErrorText;
		return false;
	}
	
	return true;
};
/**
 * @class AS.ARC.apps.radiusserver.radiusClient.radiusClient
 * @extends AS.ARC.apps.radiusserver.page
 * @author JerryJiang 2017-06-23
 */
Ext.define('AS.ARC.apps.radiusserver.radiusClient.radiusClient', {
	extend: 'AS.ARC.apps.radiusserver.page',

	pageTag: 'radiusClient',
	apiUrl: AS.ARC.util.getUserAppsPath() + 'radiusserver/radiusserver.cgi',
	SINGLE_HOST: 1,
	SUBNET: 2,

	handleApply: Ext.emptyFn,

	getRadiusClientForm: function(buttonApply) {
		var page = this;

		var onNetworkChange = function(f) {
			var v = f.getGroupValue(),
				tf = panel.down('[itemId=radius-client-subnet-mask]');
			if (v === page.SINGLE_HOST) {
				tf.clearInvalid();
				tf.disable();
			} else if (v === page.SUBNET) {
				tf.enable();
				tf.validate();
			}
		};

		var panel = Ext.create('AS.ARC.panelTop10', {
			buttonApply: buttonApply,
			defaults: {
				msgTarget: AS.ARC.config.msgTarget,
				labelWidth: AS.ARC.config.layout.defaultLabelWidth,
				blankText: _S('MESSAGE', 'HF00011'),
				width: 350
			},
			height: 490,
			items: [{
				xtype: 'displayfield',
				value: _S('USER', 'ENTER_INFO'),
				cls: 'displayField'
			}, {
				xtype: 'textfield',
				itemId: 'radius-client-name',
				fieldLabel: _S('COMMON', 'NAME'),
				allowBlank: false,
				enforceMaxLength: true,
				maxLength: 32,
				emptyText: Ext.String.format(_S('MESSAGE', 'LIMITED_LENGTH'), '32'),
				vtype: 'radiusclientname',
			}, {
				xtype: 'textfield',
				itemId: 'radius-client-shared-key',
				fieldLabel: _S('RADIUSSERVER', 'SHARED_KEY'),
				allowBlank: false,
				validator: function(v) {
					var MIN_LENGTH = 8,
						MAX_LENGTH = 32;
					if (v === '') {
						return true;
					} else if (v.length > MAX_LENGTH || v.length < MIN_LENGTH) {
						return Ext.String.format(_S('RADIUSSERVER', 'SHARED_KEY_FORMAT'), MIN_LENGTH, MAX_LENGTH);
					}
					return true;
				}
			}, {
				xtype: 'radiogroup',
				itemId: 'radiogroup',
				items: [{
					name: 'network',
					itemId: 'single-host',
					boxLabel: _S('SHARE_FOLDER', 'SINGLE_HOST'),
					inputValue: 1,
					checked: true,
					handler: onNetworkChange
				}, {
					name: 'network',
					itemId: 'subnet',
					boxLabel: _S('RADIUSSERVER', 'SUBNET'),
					inputValue: 2,
					handler: onNetworkChange
				}]
			}, {
				xtype: 'textfield',
				itemId: 'radius-client-ip-address',
				fieldLabel: _S('WEBINSTALL', 'IP_ADDRESS'),
				allowBlank: false,
				vtype: 'radiusclientipv4'
			}, {
				xtype: 'asIpv4Netmask',
				itemId: 'radius-client-subnet-mask',
				fieldLabel: _S('WEBINSTALL', 'SUBNET_MASK'),
				allowBlank: false,
				disabled: true
			}]
		});
		return panel;
	},

	handleAdd: function() {
		var page = this,
			form = {};

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			scale: AS.ARC.config.defaultApplyButtonSize,
			handler: function() {
				var urlOpts = {
						"tab": 'client',
						"act": 'add'
					},
					params = {
						"name": form.name.getValue(),
						"key": form.key.getValue(),
						"ip": form.ip.getValue(),
						"subnet": (form.group.getValue().network === page.SINGLE_HOST) ? '' : form.subnet.getValue()
					};
				win.el.mask(_S('COMMON', 'LOADING'));
				/**
				 * Add a client
				 * @param {String} tab: tab name
				 * @param {String} act: action value
				 * @param {String} name: client name
				 * @param {String} key: shared key
				 * @param {String} ip: ip address
				 * @param {String} subnet: subnet mask address (optional)
				 */
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
					method: 'post',
					params: params,
					success: function(json, opts) {
						win.close();
						page.radiusClientGrid.store.load();
					},
					failure: function(json, opts) {
						win.el.unmask();
						AS.ARC.util.showMsgWindow({
							5044: _S('MESSAGE', 'HN00020')
						}, json, win, 'warning');
					}
				});
			}
		});

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'ADD'),
			width: 450,
			height: 350,
			parentWin: page.fn.app.win,
			resizable: false,
			closable: false,
			scope: this,
			fbar: [
				buttonApply,
				{
					text: _S('COMMON', 'CANCEL'),
					handler: function() {
						win.close();
					}
				}],
			items: [page.getRadiusClientForm(buttonApply)],
			listeners: {
				afterrender: function(me) {
					form.name = me.down('[itemId=radius-client-name]');
					form.key = me.down('[itemId=radius-client-shared-key]');
					form.ip = me.down('[itemId=radius-client-ip-address]');
					form.subnet = me.down('[itemId=radius-client-subnet-mask]');
					form.group = me.down('[itemId=radiogroup]');
				}
			}
		});

		win.show();
	},

	handleEdit: function() {
		var page = this,
			form = {},
			old = {},
			selections = page.radiusClientGrid.getSelectionModel().selected;
		
		if (selections.length === 0) return;

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			scale: AS.ARC.config.defaultApplyButtonSize,
			handler: function() {
				var urlOpts = {
						"tab": 'client',
						"act": 'modify'
					},
					params = {
						"oldname": old.name,
						"name": form.name.getValue(),
						"key": form.key.getValue(),
						"ip": form.ip.getValue(),
						"subnet": (form.group.getValue().network === page.SINGLE_HOST) ? '' : form.subnet.getValue()
					};
				win.el.mask(_S('COMMON', 'LOADING'));
				/**
				 * Modify a client
				 * @param {String} tab: tab name
				 * @param {String} act: action value
				 * @param {String} name: client name
				 * @param {String} key: shared key
				 * @param {String} ip: ip address
				 * @param {String} subnet: subnet mask address (optional)
				 */
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
					method: 'post',
					params: params,
					success: function(json, opts) {
						win.close();
						page.radiusClientGrid.store.load();
					},
					failure: function(json, opts) {
						win.el.unmask();
						AS.ARC.util.showMsgWindow({
							5044: _S('MESSAGE', 'HN00020')
						}, json, win, 'warning');
					}
				});
			}
		});

		var fillInForm = function() {
			var data = selections.items[0].data;

			// save the orginial data.name
			form.name.old = old.name = data.name;
			form.ip.old = data.ip;

			form.name.setValue(data.name);
			form.key.setValue(data.key);
			form.ip.setValue(data.ip);
			if (data.subnet) {
				form.group.setValue({ "network": page.SUBNET });
				form.subnet.setValue(data.subnet);
			} else {
				form.group.setValue({ "network": page.SINGLE_HOST });
			}
		};

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'EDIT'),
			width: 450,
			height: 350,
			parentWin: page.fn.app.win,
			resizable: false,
			closable: false,
			scope: this,
			fbar: [
				buttonApply,
				{
					text: _S('COMMON', 'CANCEL'),
					handler: function() {
						win.close();
					}
				}],
			items: [page.getRadiusClientForm(buttonApply)],
			listeners: {
				afterrender: function(me) {
					form.name = me.down('[itemId=radius-client-name]');
					form.key = me.down('[itemId=radius-client-shared-key]');
					form.ip = me.down('[itemId=radius-client-ip-address]');
					form.subnet = me.down('[itemId=radius-client-subnet-mask]');
					form.group = me.down('[itemId=radiogroup]');
					fillInForm();
				}
			}
		});

		win.show();
	},

	handleEnable: function() { this.changeState(1); },

	handleDisable: function() { this.changeState(0); },

	changeState: function(enable) {
		var page = this,
			selections = page.radiusClientGrid.getSelectionModel().selected,
			urlOpts = {
				"tab": 'client',
				"act": 'edit'
			},
			params = { "enable": enable },
			name = [];
		if (selections.length === 0) return;

		Ext.each(selections.items, function(item) { name.push(item.data.name); });
		Ext.apply(params, { "name": JSON.stringify(name) });
		page.radiusClientGrid.el.mask(_S('COMMON', 'LOADING'));
		/**
		 * Enable a client
		 * @param {String} tab: tab name
		 * @param {String} act: action value
		 * @param {Number} enable: 1 / 0
		 * @param {Array} name: name list
		 */
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
			method: 'post',
			params: params,
			success: function(json, opts) {
				page.radiusClientGrid.store.load();
			},
			failure: function(json, opts) {
				page.radiusClientGrid.el.unmask();
				AS.ARC.util.showMsgWindow({}, json, page.fn.app.win, 'warning');
			}
		});
	},

	handleRemove: function() {
		var page = this,
			urlOpts = {
				"tab": 'client',
				"act": 'del'
			};

		var doRemove = function() {
			var selections = page.radiusClientGrid.getSelectionModel().selected,
				params = {},
				name = [];
			if (selections.length === 0) return;
			Ext.each(selections.items, function(item) { name.push(item.data.name); });
			Ext.apply(params, { "name": JSON.stringify(name) });
			page.radiusClientGrid.el.mask(_S('COMMON', 'LOADING'));
			/**
			 * Remove selected client(s)
			 * @param {String} tab: tab name
			 * @param {String} act: action name
			 * @param {String} name: name list
			 */
			AS.ARC.ajax({
				url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
				method: 'post',
				params: params,
				success: function (json, opts) {
					page.radiusClientGrid.store.load();
				},
				failure: function (json, opts) {
					page.radiusClientGrid.el.unmask();
					AS.ARC.util.showMsgWindow({}, json, page.fn.app.win, 'warning');
				}
			});
		};

		var win = new AS.ARC.msgWindow({
			title: _S('COMMON', 'CONFIRMATION'),
			width: 400,
			parentWin: page.fn.app.win,
			iconType: 'question',
			asItems: [{
				xtype: 'displayfield',
				value: _S('MESSAGE', 'PM00002')
			}],
			fbar: [{
				text: _S('COMMON', 'OK'),
				handler: function () {
					win.close();
					doRemove();
				},
				scope: this
			}, {
				text: _S('COMMON', 'CANCEL'),
				handler: function () {
					win.close();
				},
				scope: this
			}]
		}).show();
	},

	getRadiusClientGrid: function() {
		var page = this,
			urlOpts = {
				"tab": 'client',
				"act": 'get'
			};

		var store = Ext.create('Ext.data.JsonStore', {
			autoDestroy: true,
			pageSize: 30,
			remoteSort: true,
			fields: [
				'name', 'ip', 'subnet', 'key', 'state'
			],
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
				reader: {
					type: 'json',
					root: 'client'
				},
				listeners: {
					exception: function(proxy, response, operation, opts) {
						if (Ext.JSON.decode(response.responseText)) {
							AS.ARC.util.showMsgWindow({}, Ext.JSON.decode(response.responseText), page.fn.app.win);
						}
					}
				}
			},
			sorters: [{ property:'name', direction:'ASC'}],
			listeners: {
				beforeload: function(store, operation) {
					var query = '',
						sort = store.sorters.items[0].property,
						asc = (store.sorters.items[0].direction === 'ASC') ? true : false;
					
					if (page.fp.down('[itemId=search]')) query = page.fp.down('[itemId=search]').getValue();
					Ext.apply(store.proxy.extraParams, {
						"query": query,
						"sort": sort,
						"asc": asc
					});

					if (store.isLoading() && store.lastOperation) {
						var requests = Ext.Ajax.requests;
						for (id in requests) {
							if (requests.hasOwnProperty(id) && requests[id].options == store.lastOperation.request) {
								Ext.Ajax.abort(requests[id]);
							}
						}
					}
					store.lastOperation = operation;
					page.radiusClientGrid.el.mask(_S('COMMON', 'LOADING'));
				},
				load: function() {
					page.radiusClientGrid.el.unmask();
					page.btns.edit.disable();
					page.btns.enable.disable();
					page.btns.disable.disable();
					page.btns.remove.disable();
				}
			}
		});

		page.radiusClientGrid = Ext.create('Ext.grid.Panel', {
			itemId: 'radius-client-grid',
			height: 373,
			forceFit: true,
			multiSelect: true,
			store: store,
			bbar: Ext.create('AS.ARC.pagingToolbar', {
				store: store,
				// doRefresh: function() {}
			}),
			viewConfig: {
				onStoreLoad: function() {
					var container = this.up().body.dom;
					container.scrollTop = 0;
					Ps.update(container);
				},
				loadMask: false,
				style: { overflow: 'auto', overflowX: 'hidden' }
			},
			columns: [{
				text: _S('COMMON', 'NAME'),
				dataIndex: 'name',
				sortable: true,
				renderer: function(value, metaData, record, rowIndex, colIndex, store) {
					metaData.tdAttr = 'data-qtip="' + value + '"';
					return value;
				}
			}, {
				text: _S('WEBINSTALL', 'IP_ADDRESS'),
				dataIndex: 'ip',
				sortable: true
			}, {
				text: _S('WEBINSTALL', 'SUBNET_MASK'),
				dataIndex: 'subnet',
				sortable: true
			}, {
				text: _S('RADIUSSERVER', 'SHARED_KEY'),
				dataIndex: 'key',
				sortable: true,
				renderer: function(value, metaData, record, rowIndex, colIndex, store) {
					metaData.tdAttr = 'data-qtip="' + value + '"';
					return value;
				}
			}, {
				text: _S('DISK', 'STATE'),
				dataIndex: 'state',
				sortable: true,
				renderer: function(value, metaData, record, rowIndex, colIndex, store) {
					return (value === '1') ? _S('SYSTEMINFORMATION', 'ENABLED') : _S('COMMON', 'DISABLED');
				}
			}],
			listeners: {
				selectionchange: function(selection, selected, eOpts) {
					page.btns.edit.setDisabled(selected.length > 1);
					page.btns.enable.enable();
					page.btns.disable.enable();
					page.btns.remove.enable();
				}
			}
		});

		return page.radiusClientGrid;
	},

	getPanel: function() {
		var page = this;
		var labelWidth = AS.ARC.config.layout.indentWidth + AS.ARC.config.layout.labelWidthSmall;

		page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		var buttonAdd = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'ADD'),
			handler: page.handleAdd,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonEdit = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'EDIT'),
			handler: page.handleEdit,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonEnable = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'ENABLE'),
			handler: page.handleEnable,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonDisable = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'DISABLE'),
			handler: page.handleDisable,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonRemove = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'REMOVE'),
			handler: page.handleRemove,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		page.btns = {
			"add": buttonAdd,
			"edit": buttonEdit,
			"enable": buttonEnable,
			"disable": buttonDisable,
			"remove": buttonRemove
		};

		return {
			xtype: 'asFnForm',
			page: page,
			itemId: page.panelItemId,
			title: _S('RADIUSSERVER', 'RADIUS_CLIENT'),
			border: false,
			cls: 'as-page-panel',
			height: '100%',
			items: [{
				xtype: 'asPanelToolbarTop',
				layout: 'hbox',
				itemId: 'toolbar-top',
				items: [
					buttonAdd,
					buttonEdit,
					buttonEnable,
					buttonDisable,
					buttonRemove,
					{
						xtype: 'tbfill'
					}, {
						xtype: 'asSearchField',
						itemId: 'search',
						enableKeyEvents: true,
						checkChangeBuffer: AS.ARC.config.keyBuffer,
						listeners: {
							change: function(field) {
								var grid = page.fp.down('[itemId=radius-client-grid]');
								grid.store.loadPage(1);
							}
						}
					}]
			},
			page.getRadiusClientGrid()],
			listeners: {
				resize: function(me) {
					var logGrid = me.down('[itemId=radius-client-grid]');
					logGrid.setHeight(me.body.getHeight() - 36);
					logGrid.setWidth(me.body.getWidth());
				}
			}
		};
	},

	loadData: function() {
		var page = this;
		page.radiusClientGrid.store.load();
	}
});

/**
 * @class AS.ARC.apps.radiusserver.radiusClient.fn
 * @extends AS.ARC.apps.radiusserver.fn
 * @author JerryJiang 2017-06-23
 */
Ext.define('AS.ARC.apps.radiusserver.radiusClient.fn', {
	extend: 'AS.ARC.apps.radiusserver.fn',

	fnTag: 'radiusClient',

	init: function(config) {
		// apis
		this.createPanel();
	},

	createPanel: function() {
		var fn = this;

		// create page
		var pageRadiusClient = new AS.ARC.apps.radiusserver.radiusClient.radiusClient({ fn: this });

		this.tabPanel = Ext.create('AS.ARC.fnTabPanel', {
			items: [
				pageRadiusClient.getPanel()
			]
		});

		this.tabPanel.enableDataConfirm();

		fn.initMainPanel();
	},

	loadData: function() {
		this.tabPanel.activeTab.page.loadData(true);
	}
});