/* Copyright (c) 2017 Asustor Inc. All rights reserved. */

/**
 * vtype of radius user name
 */
Ext.form.VTypes.radiususernameMask = /[^`#$%\*+=\[\]|\\:;"'<>,?\/]/;
Ext.form.VTypes.radiususernameText = '';
Ext.form.VTypes.radiususername = function(v, f) {
	var MAX_LENGTH = 32,
		isNewName = (f.old !== v);

	if (typeof f.ajaxError === 'undefined') {
		f.ajaxError = true;
		f.ajaxErrorText = '';
	}
	
	if (!AS.ARC.re.userName.test(v)) { // check name format
		Ext.form.VTypes.radiususernameText = Ext.String.format(_S('MESSAGE', 'HF00022'), ' # $ % * + = [ ] | \\ : ; " \' < > , ? / ') + '<br>' + 
			Ext.String.format(_S('MESSAGE', 'HF00021'), '@ -') + '<br>' + 
			_S('MESSAGE', 'HF00020');
		return false;
		
	} else if (AS.ARC.util.getStringByteLength(v) > MAX_LENGTH) { // check name length 
		Ext.form.VTypes.radiususernameText = Ext.String.format(_S('ERROR', 'MAX_N_BYTE_LENGTH'), MAX_LENGTH);
		return false;
		
	} else if (v !== '' && (f.workName !== v)) {
		f.workName = v;
		f.ajaxError = false;
		if (!isNewName) {
			return true;
		}
		/**
		 * Check if name exist
		 * @param {String} act: action value.
		 * @param {String} tab: tab name.
		 * @param {String} name: User name would be checked.
		 */
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(AS.ARC.util.getUserAppsPath() + 'radiusserver/radiusserver.cgi', {
				"tab": 'user',
				"act": 'check'
			}),
			method: 'post',
			params: { "name": v },
			success: function (json, opts) {
				if (v === f.workName) {
					if (json.alreadyexists) {
						f.ajaxError = true;
						f.ajaxErrorText = _S('MESSAGE', 'HN00002');
						f.markInvalid(f.ajaxErrorText);
					} else {
						f.clearInvalid();
						f.ajaxError = false;
						f.ajaxErrorText = '';
					}
				}
			},
			failure: function (json, opts) {
				if (f.workName === v) {
					f.ajaxError = true;
					
					if (json.error_code === '5008') {
						f.ajaxErrorText = _S('MESSAGE', 'HN00002');
					} else {
						f.ajaxErrorText = 'Ref. ' + json.error_code;
					}
					f.markInvalid(f.ajaxErrorText);
				}
			}
		});
		return true;
	}
	
	if (f.ajaxError && f.ajaxErrorText != '') {
		Ext.form.VTypes.radiususernameText = f.ajaxErrorText;
		return false;
	}
	
	return true;
};

/**
 * @class AS.ARC.apps.radiusserver.radiusUser.radiusUser
 * @extends AS.ARC.apps.radiusserver.page
 * @author JerryJiang 2017-06-23
 */
Ext.define('AS.ARC.apps.radiusserver.radiusUser.radiusUser', {
	extend: 'AS.ARC.apps.radiusserver.page',

	pageTag: 'radiusUser',
	apiUrl: AS.ARC.util.getUserAppsPath() + 'radiusserver/radiusserver.cgi',
	apiUploadUrl: AS.ARC.util.getUserAppsPath() + 'radiusserver/radiusserverupload.cgi',
	isLocalUser: {
		'radius': false,
		'local': true
	},

	handleApply: Ext.emptyFn,

	getRadiusUserForm: function(buttonApply) {
		var page = this;

		var panel = Ext.create('AS.ARC.panelTop10', {
			buttonApply: buttonApply,
			defaults: {
				msgTarget: AS.ARC.config.msgTarget,
				labelWidth: AS.ARC.config.layout.defaultLabelWidth,
				checkChangeBuffer: AS.ARC.config.keyBuffer,
				blankText: _S('MESSAGE', 'HF00011'),
				width: 350
			},
			height: 490,
			items: [{
				xtype: 'textfield',
				itemId: 'radius-user-name',
				fieldLabel: _S('COMMON', 'NAME'),
				allowBlank: false,
				enforceMaxLength: true,
				maxLength: 32,
				emptyText: Ext.String.format(_S('MESSAGE', 'LIMITED_LENGTH'), '32'),
				vtype: 'radiususername',
			}, {
				xtype: 'textfield',
				itemId: 'password',
				fieldLabel: _S('COMMON', 'PWD'),
				allowBlank: false,
				inputType: 'password',
				allowBlank: false,
				validator: function(v) {
					var MIN_LENGTH = 8,
						MAX_LENGTH = 32;
					if (v === '') {
						return true;
					} else if (v.length > MAX_LENGTH || v.length < MIN_LENGTH) {
						return Ext.String.format(_S('MESSAGE', 'HF00015'), MIN_LENGTH, MAX_LENGTH);
					} else if (!/^[0-9a-zA-Z\-_\.]*$/.test(v)) {
						return Ext.String.format(_S('MESSAGE', 'HF00018'), _S('COMMON', 'COLON'), "0-9, a-z, A-Z, '.', '_', '-'");
					}
					return true;
				},
				listeners: {
					change: function (f) {
						var confirmField = f.up().items.get('password-confirm');
						
						confirmField.allowBlank = (f.value == '');
						confirmField.validate();
					}
				}
			}, {
				xtype: 'textfield',
				itemId: 'password-confirm',
				fieldLabel: _S('COMMON', 'CONFIRM_PWD'),
				inputType: 'password',
				vtype: 'userPwd'
			}]
		});
		return panel;
	},

	handleAdd: function() {
		var page = this,
			form = {};

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			scale: AS.ARC.config.defaultApplyButtonSize,
			handler: function() {
				var urlOpts = {
						"tab": 'user',
						"act": 'add'
					},
					params = {
						"name": form.name.getValue(),
						"key": form.key.getValue()
					};
				win.el.mask(_S('COMMON', 'LOADING'));
				/**
				 * Add a user
				 * @param {String} tab: tab name
				 * @param {String} act: action value
				 * @param {String} name: user name
				 * @param {String} key: password
				 */
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
					method: 'post',
					params: params,
					success: function(json, opts) {
						win.close();
						page.radiusUserGrid.store.load();
					},
					failure: function(json, opts) {
						win.el.unmask();
						AS.ARC.util.showMsgWindow({
							5044: _S('MESSAGE', 'HN00020')
						}, json, win, 'warning');
					}
				});
			}
		});

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'ADD'),
			width: 450,
			height: 250,
			parentWin: page.fn.app.win,
			resizable: false,
			closable: false,
			scope: this,
			fbar: [
				buttonApply,
				{
					text: _S('COMMON', 'CANCEL'),
					handler: function() {
						win.close();
					}
				}],
			items: [page.getRadiusUserForm(buttonApply)],
			listeners: {
				afterrender: function(me) {
					form.name = me.down('[itemId=radius-user-name]');
					form.key = me.down('[itemId=password]');
					form.confirm = me.down('[itemId=password-confirm]');
				}
			}
		});

		win.show();
	},

	handleEdit: function() {
		var page = this,
			form = {},
			old = {},
			selections = page.radiusUserGrid.getSelectionModel().selected;

		if (selections.length === 0) return;

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			scale: AS.ARC.config.defaultApplyButtonSize,
			handler: function() {
				var urlOpts = {
						"tab": 'user',
						"act": 'modify'
					},
					params = {
						"oldname": old.name,
						"name": form.name.getValue()
					};
				if (old.key !== form.key.getValue()) Ext.apply(params, { "key": form.key.getValue() });
				win.el.mask(_S('COMMON', 'LOADING'));
				/**
				 * Modify a client
				 * @param {String} tab: tab name
				 * @param {String} act: action value
				 * @param {String} oldname: orginial user name
				 * @param {String} name: user name
				 * @param {String} key: password (optional)
				 */
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
					method: 'post',
					params: params,
					success: function(json, opts) {
						win.close();
						page.radiusUserGrid.store.load();
					},
					failure: function(json, opts) {
						win.el.unmask();
						AS.ARC.util.showMsgWindow({
							5044: _S('MESSAGE', 'HN00020')
						}, json, win, 'warning');
					}
				});
			}
		});

		var fillInForm = function() {
			var data = selections.items[0].data;

			// save orginial data.name and passwordMask
			form.name.old = old.name = data.name;
			old.key = AS.ARC.config.passwordMask;

			form.name.setValue(data.name);
			form.key.setValue(AS.ARC.config.passwordMask);
			form.confirm.setValue(AS.ARC.config.passwordMask);
		};

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'EDIT'),
			width: 450,
			height: 250,
			parentWin: page.fn.app.win,
			resizable: false,
			closable: false,
			scope: this,
			fbar: [
				buttonApply,
				{
					text: _S('COMMON', 'CANCEL'),
					handler: function() {
						win.close();
					}
				}],
			items: [page.getRadiusUserForm(buttonApply)],
			listeners: {
				afterrender: function(me) {
					form.name = me.down('[itemId=radius-user-name]');
					form.key = me.down('[itemId=password]');
					form.confirm = me.down('[itemId=password-confirm]');
					fillInForm();
				}
			}
		});

		win.show();
	},

	handleEnable: function() { this.changeState(1) },

	handleDisable: function() { this.changeState(0) },

	changeState: function(enable) {
		var page = this,
			selections = page.radiusUserGrid.getSelectionModel().selected,
			urlOpts = {
				"tab": 'user',
				"act": 'edit'
			},
			params = { "enable": enable },
			name = [];
		if (selections.length === 0) return;

		Ext.each(selections.items, function(item) { name.push(item.data.name); });
		Ext.apply(params, { "name": JSON.stringify(name) });
		page.radiusUserGrid.el.mask(_S('COMMON', 'LOADING'));
		/**
		 * Enable a user
		 * @param {String} tab: tab name
		 * @param {String} act: action value
		 * @param {Number} enable: 1 / 0
		 * @param {Array} name: name list
		 */
		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
			method: 'post',
			params: params,
			success: function(json, opts) {
				page.radiusUserGrid.store.load();
			},
			failure: function(json, opts) {
				page.radiusUserGrid.el.unmask();
				AS.ARC.util.showMsgWindow({}, json, page.fn.app.win, 'warning');
			}
		});
	},

	handleRemove: function() {
		var page = this,
			urlOpts = {
				"tab": 'user',
				"act": 'del'
			};

		var doRemove = function() {
			var selections = page.radiusUserGrid.getSelectionModel().selected,
				params = {},
				name = [];
			if (selections.length === 0) return;
			Ext.each(selections.items, function(item) { name.push(item.data.name); });
			Ext.apply(params, { "name": JSON.stringify(name) });
			page.radiusUserGrid.el.mask(_S('COMMON', 'LOADING'));
			/**
			 * Remove selected user(s)
			 * @param {String} tab: tab name
			 * @param {String} act: action name
			 * @param {String} name: name list
			 */
			AS.ARC.ajax({
				url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
				method: 'post',
				params: params,
				success: function (json, opts) {
					page.radiusUserGrid.store.load();
				},
				failure: function (json, opts) {
					page.radiusUserGrid.el.unmask();
					AS.ARC.util.showMsgWindow({}, json, page.fn.app.win, 'warning');
				}
			});
		};

		var win = new AS.ARC.msgWindow({
			title: _S('COMMON', 'CONFIRMATION'),
			width: 400,
			parentWin: page.fn.app.win,
			iconType: 'question',
			asItems: [{
				xtype: 'displayfield',
				value: _S('MESSAGE', 'PM00002')
			}],
			fbar: [{
				text: _S('COMMON', 'OK'),
				handler: function () {
					win.close();
					doRemove();
				},
				scope: this
			}, {
				text: _S('COMMON', 'CANCEL'),
				handler: function () {
					win.close();
				},
				scope: this
			}]
		}).show();
	},

	handleImport: function() {
		var page = this;

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			scale: AS.ARC.config.defaultApplyButtonSize,
			handler: function() {
				win.el.mask(_S('COMMON', 'LOADING'));
				form.getForm().submit({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUploadUrl, {
						act: 'upload'
					}),
					method: 'POST',
					success: function() {
						page.handlePreview({
							parentWin: win,
							form: form
						});
					},
					failure: function(fp, o) {
						var json = Ext.JSON.decode(o.response.responseText);
						AS.ARC.util.showMsgWindow({
							5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
							5007: Ext.String.format(_S('USER', 'UPLOAD_TOO_BIG'), 1),
							5043: _S('ERROR', 'NOT_SUFFICIENT_CAPACITY_USER_CLEAR'),
							5605: _S('ERROR', 'NOT_SUFFICIENT_CAPACITY_USER_CLEAR'),
							5041: _S('VIEWER', 'UNSUPPORTED_FORMAT'),
							5016: _S('USER', 'FILE_UTF8_ONLY'),
							5026: _S('ERROR', 'UNABLE_ADD_USER'),
							5401: _S('ERROR', 'UNABLE_ADD_USER'),
							5042: _S('ERROR', 'UPLOAD_FAILED'),
							5025: _S('ERROR', 'UPLOAD_FAILED'),
							6207: _S('ERROR', 'UPLOAD_FAILED'),
							5011: _S('ERROR', 'UPLOAD_FAILED'),
							5027: _S('ERROR', 'UPLOAD_FAILED'),
							5052: _S('ERROR', 'UPLOAD_FAILED')
						}, json, win);
					}
				});
			}
		});

		var form = Ext.create('AS.ARC.panelTop10', {
			buttonApply: buttonApply,
			defaults: {
				msgTarget: AS.ARC.config.msgTarget,
				labelWidth: AS.ARC.config.layout.defaultLabelWidth,
				checkChangeBuffer: AS.ARC.config.keyBuffer,
				blankText: _S('MESSAGE', 'HF00011'),
				width: 400
			},
			height: 490,
			items: [{
				xtype: 'displayfield',
				value: _S('WINDOWS', 'FILL_OUT_BELOW')
			}, {
				xtype: 'filefield',
				fieldLabel: _S('USER', 'UPLOAD_FILE') + ' (csv, txt)',
				name: 'import-users',
				allowBlank: false,
				msgTarget: AS.ARC.config.msgTarget,
				buttonText: _S('COMMON', 'BROWSE'),
				listeners: {
					change: function(filefield, value) {
						if (filefield.value !== '') {
							filefield.setRawValue(value.replace(/C:\\fakepath\\/, ''));
						}
					}
				}
			}, {
				xtype: 'combo',
				itemId: 'user-exists-act',
				fieldLabel: _S('USER', 'IF_USER_EXISTS'),
				store: new Ext.data.ArrayStore({
					fields: ['display', 'value'],
					data: [
						[_S('FTPBACKUP', 'OVERWRITE'), 1],
						[_S('FTPBACKUP', 'SKIP'), 0]
					]
				}),
				displayField: 'display',
				valueField: 'value',
				queryMode: 'local',
				triggerAction: 'all',
				editable: false,
				value: 1
			}]
		});

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'ADD'),
			width: 450,
			height: 250,
			parentWin: page.fn.app.win,
			resizable: false,
			closable: false,
			scope: this,
			fbar: [
				buttonApply, {
					text: _S('COMMON', 'CANCEL'),
					handler: function() {
						win.close();
					}
				}],
			items: [form]
		});

		win.show();
	},

	handlePreview: function(config) {
		var page = this,
			PAGE_SIZE_10 = 10,
			PAGE_SIZE_30 = 30,
			PAGE_SIZE_50 = 50,
			PAGE_SIZE_100 = 100,
			PAGE_SIZE_500 = 500,
			overwrite = config.parentWin.down('#user-exists-act').getValue();

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			handler: function () {
				win.el.mask(_S('COMMON', 'LOADING'));
				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, { act: 'import', tab: 'user' }),
					timeout: 60 * 1000,
					params: {
						overwrite: overwrite
					},
					success: function (json) {
						win.close();
						config.parentWin.close();
						page.showImportResult(json);
						page.loadData();
					},
					failure: function (json, opts) {
						AS.ARC.util.showMsgWindow({
							5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
							5007: Ext.String.format(_S('USER', 'UPLOAD_TOO_BIG'), 1),
							5043: _S('ERROR', 'NOT_SUFFICIENT_CAPACITY_USER_CLEAR'),
							5605: _S('ERROR', 'NOT_SUFFICIENT_CAPACITY_USER_CLEAR'),
							5041: _S('VIEWER', 'UNSUPPORTED_FORMAT'),
							5016: _S('USER', 'FILE_UTF8_ONLY'),
							5026: _S('ERROR', 'UNABLE_ADD_USER'),
							5401: _S('ERROR', 'UNABLE_ADD_USER'),
							5042: _S('ERROR', 'UPLOAD_FAILED'),
							5025: _S('ERROR', 'UPLOAD_FAILED'),
							6207: _S('ERROR', 'UPLOAD_FAILED'),
							5011: _S('ERROR', 'UPLOAD_FAILED'),
							5027: _S('ERROR', 'UPLOAD_FAILED'),
							5052: _S('ERROR', 'UPLOAD_FAILED')
						}, json, win);
					}
				});
			}
		});
		
		var storePreview = Ext.create('Ext.data.JsonStore', {
			autoDestroy: true,
			pageSize: PAGE_SIZE_30,
			remoteSort: true,
			fields: ['number', 'name', 'password', 'status'],
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, { act: 'preview', tab: 'user' }),
				reader: {
					type: 'json',
					root: 'data',
					totalProperty: 'total'
				},
				listeners: {
					exception: function (proxy, response, operation, opts) {
						if (Ext.JSON.decode(response.responseText)) {
							win.close();
							AS.ARC.util.showMsgWindow({
								5000: _S('ERROR', 'CONNECTION_TIMEOUT'),
								5005: _S('USER', 'UPLOAD_FILE_EMPTY'),
								5007: _S('USER', 'MAXIMUM_ROW_NUMBER'),
								5041: _S('VIEWER', 'UNSUPPORTED_FORMAT'),
								5401: _S('USER', 'UNABLE_TO_GET_PREVIEW_INFO')
							}, Ext.JSON.decode(response.responseText), config.parentWin);
							
						}
					}
				}
			},
			sorters: [{ property:'number', direction:'ASC'}],
			listeners: {
				beforeload: function (store, operation) {
					gridPreview.el.mask(_S('COMMON', 'LOADING'));
					// TODO: sort and search
					var key,
						sortItems = store.sorters.items[0],
						sortway,
						dirway;
					
					switch(sortItems.property) {
						case 'number':
							sortway = 0;
							break;
						case 'name':
							sortway = 1;
							break;
						case 'password':
							sortway = 2;
							break;
						case 'status':
							sortway = 7;
							break;
						default:
							sortway = -1
							break;
					}
					
					if (sortItems.direction === 'ASC') {
						dirway = 1
					} else {
						dirway = 0
					}
					
					
					var sortParams = {
						sort: sortway,
						ascend: dirway
					};
					Ext.apply(store.proxy.extraParams, sortParams);

					if (store.isLoading() && store.lastOperation) {
						var requests = Ext.Ajax.requests;

						for (id in requests) {
							if (requests.hasOwnProperty(id) && requests[id].options == store.lastOperation.request) {
								Ext.Ajax.abort(requests[id]);
							}
						}
					}
					store.lastOperation = operation;
				},
				load: function(store) {
					gridPreview.el.unmask();
					var jsonData = store.proxy.reader.jsonData,
						estimatedResult = win.down('#estimated-result'),
						hint = win.down('#hint'),
						overwrite = config.parentWin.down('#user-exists-act').getValue(),
						estimateFormat = '{0} - {1}; {2} - {3}; {4} - {5} ({6}: {7})';

					estimatedResult.setValue(Ext.String.format(estimateFormat,
						_S('FILE_EXPLORER', 'COLUMN_CREATE_DATE'), jsonData.ok_count,
						((overwrite === 1) ? _S('FTPBACKUP', 'OVERWRITE') : _S('FTPBACKUP', 'SKIP')), jsonData.exist_count,
						_S('COMMON', 'FAILED'), jsonData.error_count,
						_S('APPCENTRAL', 'TOTAL'), jsonData.total
					));

					buttonApply.setDisabled(
						(jsonData.total === jsonData.error_count) ||
						(overwrite === 1 && jsonData.ok_count === 0 && jsonData.exist_count === 0) ||
						(overwrite === 0 && jsonData.ok_count === 0)
					);
					hint.setVisible(
						(overwrite === 1 && jsonData.ok_count === 0 && jsonData.exist_count === 0) ||
						(overwrite === 0 && jsonData.ok_count === 0)
					);
				}
			}
		});

		var gridPreview = Ext.create('Ext.grid.Panel', {
			height: 340,
			forceFit: true,
			store: storePreview,
			autoScroll: true,
			cls: 'as-grid',
			border: false,
			bbar: Ext.create('AS.ARC.pagingToolbar', {
				border: false,
				store: storePreview
			}),
			columns: [{
				xtype: 'gridcolumn',
				dataIndex: 'number',
				flex: 1,
				text: _S('USER', 'ROW'),
				align: 'center',
				renderer: function(value, metadata, record) {
					metadata.tdAttr = 'data-qtip="' + value + '"';
					return value;
				}
			}, {
				xtype: 'gridcolumn',
				dataIndex: 'name',
				flex: 4,
				text: _S('COMMON', 'NAME'),
				renderer: function(value, metadata, record) {
					metadata.tdAttr = 'data-qtip="' + value + '"';

					if (record.get('status') === 3 || record.get('status') === 5) {
						return '<span style="color: red;">' + value + '</span>';
					}
					return value;
				}
			}, {
				xtype: 'gridcolumn',
				dataIndex: 'password',
				flex: 4,
				text: _S('COMMON', 'PWD'),
				renderer: function(value, metadata, record) {
					metadata.tdAttr = 'data-qtip="' + value + '"';
					if (record.get('status') === 4 || record.get('status') === 6) {
						return '<span style="color: red;">' + value + '</span>';
					}
					return value;
				}
			}, {
				xtype: 'gridcolumn',
				dataIndex: 'status',
				flex: 4,
				text: _S('COMMON', 'STATUS'),
				renderer: function(value, metadata, record) {
					if (value === 1) {
						return '';
					} else if (value === 2) {
						return (overwrite === 1) ? _S('FTPBACKUP', 'OVERWRITE') : _S('FTPBACKUP', 'SKIP');
					} else if (value === 3) {
						metadata.tdAttr = 'data-errorqtip="' + _S('COMMON', 'NAME')+ ': ' + _S('USER', 'NAME_LEFT_BLANK') + '"';
						return '<a href="#" style="color: red;">' + _S('COMMON', 'ERROR') + '</a>';
					} else if (value === 4) {
						metadata.tdAttr = 'data-errorqtip="' + _S('COMMON', 'PWD') + ': ' + _S('USER', 'CANNOT_BE_BLANK') + '"';
						return '<a href="#" style="color: red;">' + _S('COMMON', 'ERROR') + '</a>';
					} else if (value === 5) {
						metadata.tdAttr = 'data-errorqtip="' + 
							Ext.htmlEncode(Ext.htmlEncode(Ext.String.format(_S('MESSAGE', 'HF00022'), ' # $ % * + = [ ] | \\ : ; " \' < > , ? / '))) + '<br>' + 
							Ext.String.format(_S('MESSAGE', 'HF00021'), '@ -') + '<br>' + 
							_S('MESSAGE', 'HF00020') + '"';
						return '<a href="#" style="color: red;">' + _S('COMMON', 'ERROR') + '</a>';
					} else if (value === 6) {
						metadata.tdAttr = 'data-errorqtip="' + Ext.String.format(_S('MESSAGE', 'HF00018'), _S('COMMON', 'COLON'), "'0-9', 'a-z', 'A-Z', '~', '!', '@', '#', '$', '%', '^', '&', '*', '.', ' '") + '"';
						return '<a href="#" style="color: red;">' + _S('COMMON', 'ERROR') + '</a>';;
					}
					return '';
				}
			}],
			listeners: {}
		});

		var buttonSize = Ext.create('Ext.button.Button', {
			text: PAGE_SIZE_30,
			checked: false,
			menu: [{
				text: PAGE_SIZE_10,
				checked: false,
				group: 'page-size',
				handler: function() {
					buttonSize.setText(PAGE_SIZE_10);
					storePreview.pageSize = PAGE_SIZE_10;
					storePreview.loadPage(1);
				}
			}, {
				text: PAGE_SIZE_30,
				checked: true,
				group: 'page-size',
				handler: function() {
					buttonSize.setText(PAGE_SIZE_30);
					storePreview.pageSize = PAGE_SIZE_30;
					storePreview.loadPage(1);
				}
			}, {
				text: PAGE_SIZE_50,
				checked: false,
				group: 'page-size',
				handler: function() {
					buttonSize.setText(PAGE_SIZE_50);
					storePreview.pageSize = PAGE_SIZE_50;
					storePreview.loadPage(1);
				}
			}, {
				text: PAGE_SIZE_100,
				checked: false,
				group: 'page-size',
				handler: function() {
					buttonSize.setText(PAGE_SIZE_100);
					storePreview.pageSize = PAGE_SIZE_100;
					storePreview.loadPage(1);
				}
			}, {
				text: PAGE_SIZE_500,
				checked: false,
				group: 'page-size',
				handler: function() {
					buttonSize.setText(PAGE_SIZE_500);
					storePreview.pageSize = PAGE_SIZE_500;
					storePreview.loadPage(1);
				}
			}]
		});

		gridPreview.dockedItems.getAt(1).add('-', {
			xtype: 'displayfield',
			value: _S('COMMON', 'SHOW'),
			margin: '0 0 0 2',
		}, buttonSize, {
			xtype: 'displayfield',
			value: _S('COMMON', 'LOW_ITEMS'),
			margin: '0 10 0 2',
		});

		var win = Ext.create('AS.ARC.subWindow', {
			parentWin: config.parentWin,
			title: _S('COMMON', 'PREVIEW'),
			width: 850,
			height: 550,
			fbar: [
				buttonApply, {
				text: _S('COMMON', 'CANCEL'),
				handler: function() {
					config.form.getForm().reset().isValid();
					win.close();
				}
			}],
			items: [{
				xtype: 'asPanelTop10',
				defaults: {
					labelWidth: AS.ARC.config.layout.labelWidthMedium
				},
				height: 500,
				items: [{
					xtype: 'asPanelToolbarTop',
					layout: 'hbox',
					hidden: true,
					items: [{
						xtype: 'tbfill'
					}, {
						xtype: 'textfield',
						listeners: {
							specialkey: function(field, e){
								if (e.getKey() == e.ENTER) {

									Ext.apply(storePreview.proxy.extraParams, {
										key: field.getValue()
									});
									storePreview.loadPage(1);
								}
							}
						}
					}]
				}, gridPreview, {
					xtype: 'displayfield',
					width: 800,
					labelWidth: AS.ARC.config.layout.labelWidthLong,
					itemId: 'estimated-result',
					fieldLabel: _S('USER', 'IMPORTED_ANALYSIS_RESULT'),
					value: AS.ARC.datas.defaultValue
				}, {
					xtype: 'displayfield',
					itemId: 'hint',
					cls: 'displayfield warning-string',
					width: 800,
					value: _S('USER', 'NO_USERS_CAN_BE_ADDED_OR_OVERWRITTEN')
				}]
			}],
			listeners: {
				afterrender: function(me) {
					gridPreview.store.load();
				}
			}
		});

		win.show();
	},

	showImportResult: function(json) {
		var page = this,
			str = '',
			resultValue = '';

		var buttonOk = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			handler: function() {
				win.close();
			}
		});

		var gridResult = Ext.create('Ext.grid.Panel', {
			height: 350,
			cls: 'appctl-installinfo',
			store: Ext.create(Ext.data.ArrayStore, {
				fields: ['status', 'result', 'type']
			}),
			autoScroll: true,
			border: false,
			columns: [{
					xtype: 'gridcolumn',
					dataIndex: 'status',
					flex: 1,
					text: _S('COMMON', 'STATUS'),
					renderer: function (value, metadata, record) {
						metadata.tdAttr = 'data-qtip="' + value + '"';
						return value;
					}
				}, {
					xtype: 'gridcolumn',
					dataIndex: 'result',
					flex: 3,
					text: _S('COMMON', 'RESULT'),
					renderer: function (value, metadata, record) {
						if (Ext.isArray(value)) {
							var link = '';
							
							switch(record.get('type')) {
								case 'exist':
									link = '<b>' + _S('FTPBACKUP', 'OVERWRITE') + ':</b> ';
									break;
								case 'owfailed':
									link= '<b>' + _S('USER', 'OVERWRITE_FAILED') + ':</b> ';
									break;
								default:
									link = '';
							}
							
							for (var i = 0; i < value.length; i ++) {
								link += '<a err=' + value[i].errcode + ' href="#" style="display: inline-block">' + value[i].name + '</a>';
								if (i !== value.length - 1) {
									link += ', ';
								}
							}
							return link;
						}
						return value;
					}
			}]
		});

		if (json.ok_count > 0) {
			Ext.each(json.ok_list, function(item, index) {
				(index === json.ok_list.length - 1) ? str += item : str += item + ', ';
			});

			resultValue = '<b>' + _S('COMMON', 'ADD') + '</b>(' + json.ok_count + '): ' + str;
			gridResult.store.add([
				[_S('COMMON', 'SUCCESS'), resultValue, 'add']
			]);
			str = '';
			resultValue = '';
		}

		if (json.exist_count > 0) {
			Ext.each(json.exist_list, function(item, index) {
				(index === json.exist_list.length - 1) ? str += item : str += item + ', ';
			});

			resultValue = ((json.overwrite === 1) ? '<b>' + _S('FTPBACKUP', 'OVERWRITE') + '</b>' : '<b>' + _S('FTPBACKUP', 'SKIP') + '</b>') + '(' + json.exist_count + '): ' + str;
			gridResult.store.add([
				[_S('COMMON', 'SUCCESS'), resultValue, 'exist']
			]);

			str = '';
			resultValue = '';
		}

		if (json.overlimit_count > 0) {
			Ext.each(json.overlimit_list, function(item, index) {
				(index === json.overlimit_list.length - 1) ? str += item : str += item + ', ';
			});

			resultValue = '<b>' + Ext.String.format(_S('USER', 'MAX_REACHED_VAR'), fn.maxUserNumber) + '</b>: ' + str;
			gridResult.store.add([
				[_S('COMMON', 'FAILED'), resultValue, 'exist']
			]);

			str = '';
			resultValue = '';
		}

		if (json.owfailed_count > 0) {
			Ext.each(json.owfailed_list, function(item, index) {
				(index === json.owfailed_list.length - 1) ? str += item.name + ' (ref. ' + item.errcode + ')' : str += item.name + ' (ref. ' + item.errcode + ')' + ', ';
			});

			resultValue = '<b>' + _S('USER', 'OVERWRITE_FAILED') + '</b>: ' + str;
			gridResult.store.add([
				[_S('COMMON', 'FAILED'), resultValue, 'owfailed']
			]);

			str = '';
			resultValue = '';
		}

		if (json.error_count > 0) {
			Ext.each(json.error_list, function(item, index) {
				(index === json.error_list.length - 1) ? str += item.name + ' (ref. ' + item.errcode + ')' : str += item.name + ' (ref. ' + item.errcode + ')' + ', ';
			});

			gridResult.store.add([
				[_S('COMMON', 'FAILED'), str, 'error']
			]);

			str = '';
			resultValue = '';
		}

		var win = Ext.create('AS.ARC.subWindow', {
			parentWin: page.fn.app.win,
			title: _S('COMMON', 'RESULT'),
			width: 800,
			height: 550,
			fbar: [buttonOk],
			items: [{
				xtype: 'asPanelTop10',
				height: 500,
				items: [{
					xtype: 'displayfield',
					value: _S('APPCENTRAL', 'REVIEW_INFO')
				}, gridResult]
			}]
		});

		win.show();
	},

	getRadiusUserGrid: function() {
		var page = this,
			urlOpts = {
				"tab": 'user',
				"act": 'get'
			};
		
		var store = Ext.create('Ext.data.JsonStore', {
			autoDestroy: true,
			pageSize: 30,
			remoteSort: true,
			fields: [
				'name', 'resource', 'state'
			],
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(page.apiUrl, urlOpts),
				reader: {
					type: 'json',
					root: 'user'
				},
				listeners: {
					exception: function(proxy, response, operation, opts) {
						if (Ext.JSON.decode(response.responseText)) {
							AS.ARC.util.showMsgWindow({}, Ext.JSON.decode(response.responseText), page.fn.app.win);
						}
					}
				}
			},
			sorters: [{ property:'name', direction:'ASC'}],
			getSortIndex: function () {},
			getSortAscend: function () {},
			listeners: {
				beforeload: function(store, operation) {
					var query = '',
						sort = store.sorters.items[0].property,
						asc = (store.sorters.items[0].direction === 'ASC') ? true : false;

					if (page.fp.down('[itemId=search]')) query = page.fp.down('[itemId=search]').getValue();
					Ext.apply(store.proxy.extraParams, {
						"query": query,
						"sort": sort,
						"asc": asc
					});

					if (store.isLoading() && store.lastOperation) {
						var requests = Ext.Ajax.requests;
						for (id in requests) {
							if (requests.hasOwnProperty(id) && requests[id].options == store.lastOperation.request) {
								Ext.Ajax.abort(requests[id]);
							}
						}
					}
					store.lastOperation = operation;
					page.radiusUserGrid.el.mask(_S('COMMON', 'LOADING'));
				},
				load: function() {
					page.radiusUserGrid.el.unmask();
					page.btns.edit.disable();
					page.btns.enable.disable();
					page.btns.disable.disable();
					page.btns.remove.disable();
				}
			}
		});

		var hasLocalUser = function() {
			var check = false,
				selections = page.radiusUserGrid.getSelectionModel().selected;
			
			Ext.each(selections.items, function(item) {
				if (page.isLocalUser[item.data.resource]) {
					check = true;
					return false;
				}
			});

			return check;
		};

		page.radiusUserGrid = Ext.create('Ext.grid.Panel', {
			itemId: 'radius-user-grid',
			height: 373,
			forceFit: true,
			multiSelect: true,
			store: store,
			bbar: Ext.create('AS.ARC.pagingToolbar', {
				store: store,
				// doRefresh: function() {}
			}),
			viewConfig: {
				onStoreLoad: function() {
					var container = this.up().body.dom;
					container.scrollTop = 0;
					Ps.update(container);
				},
				loadMask: false,
				style: { overflow: 'auto', overflowX: 'hidden' }
			},
			columns: [{
				text: _S('COMMON', 'NAME'),
				dataIndex: 'name',
				sortable: true,
				renderer: function(value, metaData, record, rowIndex, colIndex, store) {
					metaData.tdAttr = 'data-qtip="' + value + '"';
					return value;
				}
			}, {
				text: _S('RADIUSSERVER', 'DATA_SOURCE'),
				dataIndex: 'resource',
				sortable: true,
				renderer: function(value, metaData, record, rowIndex, colIndex, store) {
					var result = '';
					result = (page.isLocalUser[value]) ? 'NAS ' : 'Radius ';
					metaData.tdAttr = 'data-qtip="' + result + _S('COMMON', 'ACCOUNT') + '"';
					return result + _S('COMMON', 'ACCOUNT');
				}
			}, {
				text: _S('DISK', 'STATE'),
				dataIndex: 'state',
				sortable: true,
				renderer: function(value, metaData, record, rowIndex, colIndex, store) {
					return (value === '1') ? _S('SYSTEMINFORMATION', 'ENABLED') : _S('COMMON', 'DISABLED');
				}
			}],
			listeners: {
				selectionchange: function(selection, selected, eOpts) {
					page.btns.edit.setDisabled(hasLocalUser() || selected.length > 1);
					page.btns.enable.setDisabled(hasLocalUser());
					page.btns.disable.setDisabled(hasLocalUser());
					page.btns.remove.setDisabled(hasLocalUser());
				}
			}
		});

		return page.radiusUserGrid;
	},

	getPanel: function() {
		var page = this;

		page.buttonApply = Ext.create('Ext.button.Button', {
			xtype: 'button',
			text: _S('COMMON', 'APPLY')
		});

		var buttonAddOneUser = Ext.create('Ext.menu.Item', {
			text: _S('USER', 'ADD_NEW_USER'),
			handler: page.handleAdd,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize,
		});

		var buttonAddUsersFromFile = Ext.create('Ext.menu.Item', {
			text: _S('USER', 'IMPORT_USERS'),
			handler: page.handleImport,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize,
		});

		var buttonAdd = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'ADD'),
			menu: [
				buttonAddOneUser,
				buttonAddUsersFromFile
			],
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonEdit = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'EDIT'),
			handler: page.handleEdit,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonEnable = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'ENABLE'),
			handler: page.handleEnable,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonDisable = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'DISABLE'),
			handler: page.handleDisable,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		var buttonRemove = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'REMOVE'),
			handler: page.handleRemove,
			scope: this,
			scale: AS.ARC.config.defaultActionButtonSize
		});

		page.btns = {
			"add": buttonAdd,
			"edit": buttonEdit,
			"enable": buttonEnable,
			"disable": buttonDisable,
			"remove": buttonRemove
		};

		return {
			xtype: 'asFnForm',
			page: page,
			itemId: page.panelItemId,
			title: _S('RADIUSSERVER', 'RADIUS_USER'),
			border: false,
			cls: 'as-page-panel',
			height: '100%',
			items: [{
				xtype: 'asPanelToolbarTop',
				layout: 'hbox',
				itemId: 'toolbar-top',
				items: [
					buttonAdd,
					buttonEdit,
					buttonEnable,
					buttonDisable,
					buttonRemove,
				{
					xtype: 'tbfill'
				}, {
					xtype: 'asSearchField',
					itemId: 'search',
					enableKeyEvents : true,
					checkChangeBuffer: AS.ARC.config.keyBuffer,
					listeners: {
						change: function (field) {
							var grid = page.fp.down('[itemId=radius-user-grid]');
							grid.store.loadPage(1);
						}
					}
				}]
			},
			page.getRadiusUserGrid()],
			listeners: {
				resize: function(me) {
					var logGrid = me.down('[itemId=radius-user-grid]');
					logGrid.setHeight(me.body.getHeight() - 36);
					logGrid.setWidth(me.body.getWidth());
				}
			}
		};
	},

	loadData: function() {
		var page = this;
		page.radiusUserGrid.store.load();
	}
});

/**
 * @class AS.ARC.apps.radiusserver.radiusUser.fn
 * @extends AS.ARC.apps.radiusserver.fn
 * @author JerryJiang 2017-06-23
 */
Ext.define('AS.ARC.apps.radiusserver.radiusUser.fn', {
	extend: 'AS.ARC.apps.radiusserver.fn',

	fnTag: 'radiusUser',

	init: function(config) {
		// apis
		this.createPanel();
	},

	createPanel: function() {
		var fn = this;

		// create page
		var pageRadiusUser = new AS.ARC.apps.radiusserver.radiusUser.radiusUser({ fn: this });

		this.tabPanel = Ext.create('AS.ARC.fnTabPanel', {
			items: [
				pageRadiusUser.getPanel()
			]
		});

		this.tabPanel.enableDataConfirm();

		fn.initMainPanel();
	},

	loadData: function() {
		this.tabPanel.activeTab.page.loadData(true);
	}
});